/************************************************************
(c)Copyright 1992 by Hewlett-Packard Co., Palo Alto, California.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Hewlett-Packard or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

HEWLETT-PACKARD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
HEWLETT-PACKARD BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

********************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/framebuf.h>
#include <fcntl.h>

#include <X.h>
#include <Xproto.h>
#include <scrnintstr.h>
#include <pixmapstr.h>
#include <resource.h>
#include <misc.h>
#include <mi.h>
#include <servermd.h>
#include <windowstr.h>
#include <colormap.h>

#include "hppriv.h"

#if defined(__hp9000s700) || defined(__hp9000s300)
/* Until <graphics.h> (s400) and <framebuf.h> (s700) are updated,
 * define graphics IDs returned by ioctl(GCDESCRIBE).  On both
 * platforms, this ID equals the upper 4 bytes of the STI ROM ID.
 *
 * (s400 won't support 24-plane color (Rattler); that's handled below).
 */
/* Until <framebuf.h> (s700) is updated */
#   ifndef S9000_ID_A1659A          /* CRX (8-plane Color) */
#       define S9000_ID_A1659A          0x26D1482A
#   endif
#   ifndef S9000_ID_A1924A          /* GRX (Gray Scale) */
#       define S9000_ID_A1924A          0x26D1488C
#   endif
#   ifndef S9000_ID_A1439A          /* CRX+ (24-plane Color */
#       define S9000_ID_A1439A          0x26D148EE
#   endif
#   ifndef S9000_ID_TIMBER          /* Bushmaster (710) Graphics */
#       define S9000_ID_TIMBER          0x27F12392
#   endif
#   ifndef S9000_ID_TOMCAT          /* 2-headed ELK; sorry, no P/N yet */
#       define S9000_ID_TOMCAT          0x27FCCB6D
#   endif
#endif

/* Declare the external initialization functions */

extern Bool	ngleScreenInit();


        /*  --------------------- ddx_driver_info -------------------------- */
        /*  This is an inital entry point into the hpux ddx drivers.  This 
	    procedure is called to determine if ddx driver support exists 
	    for a given display and configuration.  The display and 
	    configuration are defined by the hp private structure passed 
	    into this routine.  A null pointer is returned if no ddx driver 
	    support is available.  If a ddx driver does exist to support the 
	    given configuration, then a pointer to the driver's InitOutput 
	    routine is returned.
        */

Bool (*ddx_driver_info( php))()
    hpPrivPtr php;
{
    struct stat 		statInfo;
    int 			result;
    crt_frame_buffer_t 		desc;
    char 			*map_origin = NULL;
    Bool			(*return_value)();


    /* Stat the file descriptor to see if the device file exists: */
    if((result = stat( php->StandardDevice, &statInfo)) < 0)
    {
	return(NULL);
    }

    /* Stat the file descriptor to see if the device exists: */
    if(!( statInfo.st_mode & S_IFCHR))
    {
	return(NULL);
    }

    /* Let's open the device: */
    if(( php->StandardFd = open(php->StandardDevice, O_RDWR)) < 0)
    {
	return(NULL);
    }

    /* Map the device: */
    if( ioctl( php->StandardFd, GCMAP, &map_origin) < 0)
    {
	close(php->StandardFd);
	return(NULL);
    }

    /* Does the kernel know what's out there?: */
    if( ioctl( php->StandardFd, GCDESCRIBE, &desc) < 0)
    {
	ioctl(php->StandardFd, GCUNMAP, &map_origin );
	close(php->StandardFd );
	return(NULL);
    }

    /* Let's give the driver a chance to look at the device: */
    switch(desc.crt_id)
    {
	/* Devices supported on s700 */
	case S9000_ID_A1924A:	    /* GRX (8-plane Gray Scale) */
	case S9000_ID_A1659A:	    /* CRX (8-plane Color) */
	case S9000_ID_TOMCAT:	    /* 2-Headed CRX */
	case S9000_ID_TIMBER:	    /* Bushmaster (710) Graphics */
	    return_value = ngleScreenInit;
	    break;

	case S9000_ID_A1439A:	    /* CRX24 (24-plane Color) */
	    return_value = NULL;
	    break;

	default:
	    return_value = NULL;
	    break;
    }

    /* We've got our info, so lets close things down so the driver can
     * re-open them:
     */
    ioctl(php->StandardFd, GCUNMAP, &map_origin);
    close(php->StandardFd );

    return(return_value);
}
