/************************************************************
(c)Copyright 1992 by Hewlett-Packard Co., Palo Alto, California.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Hewlett-Packard or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

HEWLETT-PACKARD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
HEWLETT-PACKARD BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

********************************************************/

#ifndef NGLEHDW_H
#define NGLEHDW_H


typedef ngle_dregs_t	NgleHdwRec;
typedef ngle_dregs_t	*NgleHdwPtr;

#define SETUP_HW(pDregs) {						\
	    char stat;							\
	    volatile char * pstat = &((pDregs)->reg15.b.b0);		\
	    do {							\
		stat = *pstat;						\
		if (!stat)						\
		    stat = *pstat;					\
	    } while(stat);						\
	}

#define SETUP_FB(pDregs,ID) {						\
	    SETUP_HW(pDregs);						\
	    switch (ID) {						\
		case S9000_ID_A1659A:					\
		    (pDregs)->reg10 = 0x13601000;			\
		    break; 						\
		case S9000_ID_TIMBER:					\
		case S9000_ID_A1439A:					\
		    (pDregs)->reg10 = 0x13602000;			\
		    break;						\
	    }								\
	    (pDregs)->reg14.all = 0x83000300;				\
	    SETUP_HW(pDregs);						\
	    (pDregs)->reg16.b.b1 = 1;					\
	}

#define START_CURSOR_COLORMAP_ACCESS(pDregs) {				\
	    SETUP_HW(pDregs);						\
	    (pDregs)->reg10 = 0xBBE0F000;				\
	    (pDregs)->reg14.all = 0x03000300;				\
	    (pDregs)->reg13 = ~0;					\
	    (pDregs)->reg3 = 0;						\
	    (pDregs)->reg4 = 0;						\
	    (pDregs)->reg4 = 0;						\
	}

#define WRITE_CURSOR_COLOR(pDregs,color) {				\
	    (pDregs)->reg4 = (color);					\
	}

#define FINISH_CURSOR_COLORMAP_ACCESS(pDregs,ID) {			\
	    (pDregs)->reg2 = 0;						\
	    (pDregs)->reg1 = 0x80008004;				\
	    SETUP_FB(pDregs,ID);					\
	}

#define START_IMAGE_COLORMAP_ACCESS(pDregs) {				\
	    SETUP_HW(pDregs);						\
	    (pDregs)->reg10 = 0xBBE0F000;				\
	    (pDregs)->reg14.all = 0x03000300;				\
	    (pDregs)->reg13 = ~0;					\
	}

#define WRITE_IMAGE_COLOR(pDregs,index,color) {				\
	    SETUP_HW(pDregs);						\
	    (pDregs)->reg3 = ((0x100+index) << 2);			\
	    (pDregs)->reg4 = (color);					\
	}

#define FINISH_IMAGE_COLORMAP_ACCESS(pDregs,ID) {			\
	    (pDregs)->reg2 = 0x400;					\
	    (pDregs)->reg1 = 0x80000100;				\
	    SETUP_FB(pDregs,ID);					\
	}

#define GET_CURSOR_SPECS(pDregs,pScreenPriv) {				      \
	    Card32 activeLinesHi, activeLinesLo;			      \
									      \
	    (pScreenPriv)->sprite.horizBackPorch = (pDregs)->reg19.b.b1;      \
	    activeLinesLo   = (pDregs)->reg20.b.b0;		      	      \
	    activeLinesHi   = ((pDregs)->reg21.b.b2) & 0xf;		      \
	    (pScreenPriv)->sprite.maxYLine = ((activeLinesHi << 8)	      \
		|(activeLinesLo & 0xff));				      \
	}

#define START_CURSOR_MASK_ACCESS(pDregs) {				\
	    SETUP_HW(pDregs);						\
	    (pDregs)->reg14.all = 0x00000300;				\
	    (pDregs)->reg13 = ~0;					\
	    (pDregs)->reg11 = 0x28A07000;				\
	    (pDregs)->reg3 = 0;						\
	}

#define START_CURSOR_DATA_ACCESS(pDregs) {				\
	    SETUP_HW(pDregs);						\
	    (pDregs)->reg14.all = 0x00000300;				\
	    (pDregs)->reg11 = 0x28A06000;				\
	    (pDregs)->reg3 = 0;						\
	}

#define WRITE_CURSOR_BITS(pDregs,word1,word2) {				\
	    SETUP_HW(pDregs);						\
	    (pDregs)->reg4 = (word1);					\
	    (pDregs)->reg5 = (word2);					\
	}

#define ENABLE_CURSOR(pScreenPriv, cursorXYValue, nOffscreenScanlines) { \
	    (pScreenPriv)->sprite.enabledCursorXYValue = (cursorXYValue);\
	    if ((pScreenPriv)->sprite.moveOnVBlank) {			 \
		while (((pScreenPriv)->pDregs->reg21.all) & 0x00040000); \
		while (!(((pScreenPriv)->pDregs->reg21.all)		 \
			& 0x00040000));					 \
	    }								 \
	    SETUP_HW(pScreenPriv->pDregs);				 \
	    (pScreenPriv)->pDregs->reg17.all = (cursorXYValue);		 \
	    (pScreenPriv)->pDregs->reg18.all =				 \
		    (0x80 | nOffscreenScanlines);			 \
	}

#define DISABLE_CURSOR(pScreenPriv) {					\
	    volatile unsigned long *pDregsCursorXY = 			\
		    &((pScreenPriv)->pDregs->reg17.all);		\
	    long enabledCursorValue = 					\
		    (pScreenPriv)->sprite.enabledCursorXYValue;		\
									\
	    SETUP_HW((pScreenPriv)->pDregs);				\
	    *pDregsCursorXY = (enabledCursorValue & 0xe007ffff);	\
	}

#define GET_ROMTABLE_INDEX(romTableIdx) {				\
	    (romTableIdx) = pDregs->reg16.b.b3 - 1;			\
	}

#define SETUP_RAMDAC(pDregs) {						\
	    volatile Card32         *pAuxControlSpace;			\
									\
	    pAuxControlSpace = (Card32 *) (((Card32) pDregs) + 0x1000); \
	    SETUP_HW(pDregs);						\
	    *(pAuxControlSpace + (0x20>>2)) = 0x04000000;		\
	    *(pAuxControlSpace + (0x28>>2)) = 0xff000000;		\
	}

#define ENABLE_DISPLAY(pDregs) {					\
	    volatile Card32         *pAuxControlSpace;			\
									\
	    pAuxControlSpace = (Card32 *)(((Card32)(pDregs)) + 0x1000); \
	    SETUP_HW(pDregs);						\
	    *(pAuxControlSpace + (0x30>>2)) = 0x06000000;		\
	    *(pAuxControlSpace + (0x38>>2)) = 0x43000000;		\
	}

#define DISABLE_DISPLAY(pDregs) {					\
	    volatile Card32         *pAuxControlSpace;			\
									\
	    pAuxControlSpace = (Card32 *)(((Card32)(pDregs)) + 0x1000); \
	    SETUP_HW(pDregs);						\
	    *(pAuxControlSpace + (0x30>>2)) = 0x06000000;		\
	    *(pAuxControlSpace + (0x38>>2)) = 0x03000000;		\
	}

#define CRX24_ENABLE_DISPLAY(pDregs) {					\
	    volatile Card32         *pAuxControlSpace;			\
									\
	    pAuxControlSpace = (Card32 *)(((Card32)(pDregs)) + 0x1000); \
	    SETUP_HW(pDregs);						\
	    *pAuxControlSpace     = 0x01000000;				\
	    *(pAuxControlSpace+1) = 0x02000000;				\
	    *(pAuxControlSpace+2) = 0x10000000;				\
	}

#define CRX24_DISABLE_DISPLAY(pDregs) {					\
	    volatile Card32         *pAuxControlSpace;			\
									\
	    pAuxControlSpace = (Card32 *)(((Card32)(pDregs)) + 0x1000); \
	    SETUP_HW(pDregs);						\
	    *pAuxControlSpace     = 0x01000000;				\
	    *(pAuxControlSpace+1) = 0x02000000;				\
	    *(pAuxControlSpace+2) = 0x30000000;				\
	}


#define BUFFER0 0x00000202
#define BUFFER1 0x02000202

#define SETUP_ATTR_ACCESS(pDregs,BufferNumber) {			\
	    SETUP_HW(pDregs);						\
	    (pDregs)->reg11 = 0x2EA0D000;				\
	    (pDregs)->reg14.all = 0x23000302;			\
	    (pDregs)->reg12 = (BufferNumber);			\
	    (pDregs)->reg8 = 0xffffffff;				\
	}

#define SET_ATTR_SIZE(pDregs,Width,Height) {				    \
	    (pDregs)->reg6.all = 0;				    \
	    (pDregs)->reg9.all = (Card32)(((Width)<<16)|(Height));\
	    (pDregs)->reg6.all = 0x05000000;			    \
	    (pDregs)->reg9.all = 0x00040001;			    \
	}

#define FINISH_ATTR_ACCESS(pDregs) {					\
	    SETUP_HW(pDregs);						\
	    (pDregs)->reg12 = 0;					\
	}


#endif /* NGLEHDW_H */
